@@ -1,7 +1,9 @@
 use std::collections::HashMap;
 
-use grep_matcher::{Captures, Match, Matcher};
-use pcre2::bytes::{CaptureLocations, Regex, RegexBuilder};
+use {
+    grep_matcher::{Captures, Match, Matcher},
+    pcre2::bytes::{CaptureLocations, Regex, RegexBuilder},
+};
 
 use crate::error::Error;
 
@@ -11,6 +13,8 @@ pub struct RegexMatcherBuilder {
     builder: RegexBuilder,
     case_smart: bool,
     word: bool,
+    fixed_strings: bool,
+    whole_line: bool,
 }
 
 impl RegexMatcherBuilder {
@@ -20,6 +24,8 @@ impl RegexMatcherBuilder {
             builder: RegexBuilder::new(),
             case_smart: false,
             word: false,
+            fixed_strings: false,
+            whole_line: false,
         }
     }
 
@@ -29,17 +35,40 @@ impl RegexMatcherBuilder {
     /// If there was a problem compiling the pattern, then an error is
     /// returned.
     pub fn build(&self, pattern: &str) -> Result<RegexMatcher, Error> {
+        self.build_many(&[pattern])
+    }
+
+    /// Compile all of the given patterns into a single regex that matches when
+    /// at least one of the patterns matches.
+    ///
+    /// If there was a problem building the regex, then an error is returned.
+    pub fn build_many<P: AsRef<str>>(
+        &self,
+        patterns: &[P],
+    ) -> Result<RegexMatcher, Error> {
         let mut builder = self.builder.clone();
-        if self.case_smart && !has_uppercase_literal(pattern) {
+        let mut pats = Vec::with_capacity(patterns.len());
+        for p in patterns.iter() {
+            pats.push(if self.fixed_strings {
+                format!("(?:{})", pcre2::escape(p.as_ref()))
+            } else {
+                format!("(?:{})", p.as_ref())
+            });
+        }
+        let mut singlepat = pats.join("|");
+        if self.case_smart && !has_uppercase_literal(&singlepat) {
             builder.caseless(true);
         }
-        let res = if self.word {
-            let pattern = format!(r"(?<!\w)(?:{})(?!\w)", pattern);
-            builder.build(&pattern)
-        } else {
-            builder.build(pattern)
-        };
-        res.map_err(Error::regex).map(|regex| {
+        if self.whole_line {
+            singlepat = format!(r"(?m:^)(?:{})(?m:$)", singlepat);
+        } else if self.word {
+            // We make this option exclusive with whole_line because when
+            // whole_line is enabled, all matches necessary fall on word
+            // boundaries. So this extra goop is strictly redundant.
+            singlepat = format!(r"(?<!\w)(?:{})(?!\w)", singlepat);
+        }
+        log::trace!("final regex: {:?}", singlepat);
+        builder.build(&singlepat).map_err(Error::regex).map(|regex| {
             let mut names = HashMap::new();
             for (i, name) in regex.capture_names().iter().enumerate() {
                 if let Some(ref name) = *name {
@@ -144,6 +173,21 @@ impl RegexMatcherBuilder {
         self
     }
 
+    /// Whether the patterns should be treated as literal strings or not. When
+    /// this is active, all characters, including ones that would normally be
+    /// special regex meta characters, are matched literally.
+    pub fn fixed_strings(&mut self, yes: bool) -> &mut RegexMatcherBuilder {
+        self.fixed_strings = yes;
+        self
+    }
+
+    /// Whether each pattern should match the entire line or not. This is
+    /// equivalent to surrounding the pattern with `(?m:^)` and `(?m:$)`.
+    pub fn whole_line(&mut self, yes: bool) -> &mut RegexMatcherBuilder {
+        self.whole_line = yes;
+        self
+    }
+
     /// Enable Unicode matching mode.
     ///
     /// When enabled, the following patterns become Unicode aware: `\b`, `\B`,
@@ -178,23 +222,22 @@ impl RegexMatcherBuilder {
         self
     }
 
-    /// When UTF matching mode is enabled, this will disable the UTF checking
-    /// that PCRE2 will normally perform automatically. If UTF matching mode
-    /// is not enabled, then this has no effect.
+    /// This is now deprecated and is a no-op.
     ///
-    /// UTF checking is enabled by default when UTF matching mode is enabled.
-    /// If UTF matching mode is enabled and UTF checking is enabled, then PCRE2
-    /// will return an error if you attempt to search a subject string that is
-    /// not valid UTF-8.
+    /// Previously, this option permitted disabling PCRE2's UTF-8 validity
+    /// check, which could result in undefined behavior if the haystack was
+    /// not valid UTF-8. But PCRE2 introduced a new option, `PCRE2_MATCH_INVALID_UTF`,
+    /// in 10.34 which this crate always sets. When this option is enabled,
+    /// PCRE2 claims to not have undefined behavior when the haystack is
+    /// invalid UTF-8.
     ///
-    /// # Safety
-    ///
-    /// It is undefined behavior to disable the UTF check in UTF matching mode
-    /// and search a subject string that is not valid UTF-8. When the UTF check
-    /// is disabled, callers must guarantee that the subject string is valid
-    /// UTF-8.
-    pub unsafe fn disable_utf_check(&mut self) -> &mut RegexMatcherBuilder {
-        self.builder.disable_utf_check();
+    /// Therefore, disabling the UTF-8 check is not something that is exposed
+    /// by this crate.
+    #[deprecated(
+        since = "0.2.4",
+        note = "now a no-op due to new PCRE2 features"
+    )]
+    pub fn disable_utf_check(&mut self) -> &mut RegexMatcherBuilder {
         self
     }
 
@@ -385,9 +428,10 @@ fn has_uppercase_literal(pattern: &str) -> bool {
 
 #[cfg(test)]
 mod tests {
-    use super::*;
     use grep_matcher::{LineMatchKind, Matcher};
 
+    use super::*;
+
     // Test that enabling word matches does the right thing and demonstrate
     // the difference between it and surrounding the regex in `\b`.
     #[test]
